from gaelib.tests.auth.base import BaseAuthUnitTestCase
from gaelib.storage import helpers
from gaelib.auth.twilio_client import TwilioClient
from gaelib.auth import auth
from mock import patch
# from main import app
import base64
from flask import session
from werkzeug.http import dump_cookie
import json


class AuthTestCase(BaseAuthUnitTestCase):

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_present_and_session_is_present(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"user:pass").decode("utf8"))
    }
    with self.tests_app.test_request_context(headers=headers):
      session['gae_uid'] = 'gae_uid'
      session['gae_token'] = 'gae_token'
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'user')
      self.assertEquals(id_token, 'pass')

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_present_and_session_is_not_present(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"user:pass").decode("utf8"))
    }
    with self.tests_app.test_request_context(headers=headers):
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'user')
      self.assertEquals(id_token, 'pass')

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_not_present_and_session_is_present(self):

    with self.tests_app.test_request_context():
      session['gae_uid'] = 'gae_uid'
      session['gae_token'] = 'gae_token'
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'gae_uid')
      self.assertEquals(id_token, 'gae_token')

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_not_present_and_session_is_not_present(self):

    with self.tests_app.test_request_context():

      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertIsNone(user_id)
      self.assertIsNone(id_token)

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_present_and_firebaseaccesstoken_is_present_and_is_dashboard_url(self):
    cookie = dump_cookie("firebaseAccessToken", 'Cookie_value')
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"user:pass").decode("utf8")),
        'COOKIE': cookie
    }
    url = 'admindashboard/login'
    with self.tests_app.test_request_context(url, headers=headers):
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'user')
      self.assertEquals(id_token, 'Cookie_value')

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_present_and_firebaseaccesstoken_is_present_and_is_not_dashboard_url(self):
    cookie = dump_cookie("firebaseAccessToken", 'Cookie_value')
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"user:pass").decode("utf8")),
        'COOKIE': cookie
    }
    with self.tests_app.test_request_context(headers=headers):
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'user')
      self.assertEquals(id_token, 'pass')

  def test_user_id_and_token_for_get_user_id_and_token_when_authorization_is_not_present_and_firebaseaccesstoken_is_present_and_is_dashboard_url(self):
    cookie = dump_cookie("firebaseAccessToken", 'Cookie_value')
    headers = {
        'COOKIE': cookie
    }
    url = 'admindashboard/login'
    with self.tests_app.test_request_context(url, headers=headers):
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, None)
      self.assertEquals(id_token, 'Cookie_value')

  def test_user_id_and_token_for_get_user_id_and_token_when_session_is_present_and_firebaseaccesstoken_is_present_and_is_dashboard_url(self):
    cookie = dump_cookie("firebaseAccessToken", 'Cookie_value')
    headers = {
        'COOKIE': cookie
    }
    url = 'admindashboard/login'
    with self.tests_app.test_request_context(url, headers=headers):
      session['gae_uid'] = 'gae_uid'
      session['gae_token'] = 'gae_token'
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'gae_uid')
      self.assertEquals(id_token, 'Cookie_value')

  def test_user_id_and_token_for_get_user_id_and_token_when_session_is_present_and_firebaseaccesstoken_is_present_and_is_not_dashboard_url(self):
    cookie = dump_cookie("firebaseAccessToken", 'Cookie_value')
    headers = {
        'COOKIE': cookie
    }
    with self.tests_app.test_request_context(headers=headers):
      session['gae_uid'] = 'gae_uid'
      session['gae_token'] = 'gae_token'
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, 'gae_uid')
      self.assertEquals(id_token, 'gae_token')

  def test_user_id_and_token_for_get_user_id_and_token_when_session_is_not_present_and_firebaseaccesstoken_is_present_and_is_dashboard_url(self):
    cookie = dump_cookie("firebaseAccessToken", 'Cookie_value')
    headers = {
        'COOKIE': cookie
    }
    url = 'admindashboard/login'
    with self.tests_app.test_request_context(url, headers=headers):
      self.tests_app.preprocess_request()
      user_id, id_token = auth.get_user_id_and_token()
      self.assertEquals(user_id, None)
      self.assertEquals(id_token, 'Cookie_value')

  def test_auth_type_when_auth_type_is_present_in_args(self):
    url = 'abc?auth_type=test_auth'
    with self.tests_app.test_request_context(url):
      self.tests_app.preprocess_request()
      auth_type = auth.get_auth_type()
      self.assertEquals(auth_type, 'test_auth')

  def test_auth_type_when_auth_type_is_present_in_json(self):
    with self.tests_app.test_request_context(json={"auth_type": "test_auth"}):
      self.tests_app.preprocess_request()
      auth_type = auth.get_auth_type()
      self.assertEquals(auth_type, 'test_auth')

  def test_auth_type_when_auth_type_is_present_in_form(self):
    with self.tests_app.test_request_context(data={"auth_type": "test_auth"}):
      self.tests_app.preprocess_request()
      auth_type = auth.get_auth_type()
      self.assertEquals(auth_type, 'test_auth')

  # def test_auth_type_for_get_user_id_and_token_when_auth_type_is_present_in_params(self):
  #   cookie = dump_cookie("auth_type", 'test_auth')
  #   headers = {
  #       'COOKIE': cookie
  #   }
  #   with self.tests_app.test_request_context(data={"auth_type": "test_auth"}, method='POST'):
  #     self.tests_app.preprocess_request()
  #     _, _, auth_type, _ = auth.get_user_id_and_token()
  #     self.assertEquals(auth_type, 'test_auth')

  def test_auth_type_when_auth_type_is_not_present(self):
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      auth_type = auth.get_auth_type()
      self.assertEquals(auth_type, 'firebase')

  def test_authorize_login_request_when_auth_type_is_absent(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_firebase_authorize()
      self.firebase_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      claims = auth_obj.authorize_login_request()
      self.assertIsNotNone(claims)
      self.assertEquals(3, len(claims))
      self.assertEquals('sub', claims['sub'])
      self.assertEquals('user', claims['name'])
      self.assertEquals('email@abc', claims['email'])

  def test_authorize_login_request_when_auth_type_is_firebase(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_firebase_authorize()
      self.firebase_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      claims = auth_obj.authorize_login_request(auth_type='firebase')
      self.assertIsNotNone(claims)
      self.assertEquals(3, len(claims))
      self.assertEquals('sub', claims['sub'])
      self.assertEquals('user', claims['name'])
      self.assertEquals('email@abc', claims['email'])

  def test_authorize_login_request_when_auth_type_is_auth0(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_auth0_authorize()
      self.auth0_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      claims = auth_obj.authorize_login_request(auth_type='auth0')
      self.assertIsNotNone(claims)
      self.assertEquals(3, len(claims))
      self.assertEquals('sub', claims['sub'])
      self.assertEquals('user', claims['name'])
      self.assertEquals('email@abc', claims['email'])

  def test_authorize_request_when_auth_type_is_absent_and_claimed_user_id_is_same_as_user_id(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_firebase_authorize()
      self.firebase_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      auth_obj.user_id = 'sub'
      claims = auth_obj.authorize_request()
      self.assertIsNotNone(claims)
      self.assertEquals(3, len(claims))
      self.assertEquals('sub', claims['sub'])
      self.assertEquals('user', claims['name'])
      self.assertEquals('email@abc', claims['email'])

  def test_authorize_request_when_auth_type_is_absent_and_claimed_user_id_is_not_same_as_user_id(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_firebase_authorize()
      self.firebase_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      auth_obj.user_id = 'sub_1'
      claims = auth_obj.authorize_request()
      self.assertEquals(len(claims), 0)

  def test_authorize_request_when_auth_type_is_firebase_and_claimed_user_id_is_same_as_user_id(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_firebase_authorize()
      self.firebase_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      auth_obj.user_id = 'sub'
      claims = auth_obj.authorize_request(auth_type='firebase')
      self.assertIsNotNone(claims)
      self.assertEquals(3, len(claims))
      self.assertEquals('sub', claims['sub'])
      self.assertEquals('user', claims['name'])
      self.assertEquals('email@abc', claims['email'])

  def test_authorize_request_when_auth_type_is_firebase_and_claimed_user_id_is_not_same_as_user_id(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_firebase_authorize()
      self.firebase_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      auth_obj.user_id = 'sub_1'
      claims = auth_obj.authorize_request(auth_type='firebase')
      self.assertEquals(len(claims), 0)

  def test_authorize_request_when_auth_type_is_auth0_and_claimed_user_id_is_same_as_user_id(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_auth0_authorize()
      self.auth0_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      auth_obj.user_id = 'sub'
      claims = auth_obj.authorize_request(auth_type='auth0')
      self.assertIsNotNone(claims)
      self.assertEquals(3, len(claims))
      self.assertEquals('sub', claims['sub'])
      self.assertEquals('user', claims['name'])
      self.assertEquals('email@abc', claims['email'])

  def test_authorize_request_when_auth_type_is_auth0_and_claimed_user_id_is_not_same_as_user_id(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.mock_auth0_authorize()
      self.auth0_authorize.return_value = return_value
      auth_obj = auth.Auth('token')
      auth_obj.user_id = 'sub_1'
      claims = auth_obj.authorize_request(auth_type='auth0')
      self.assertEquals(len(claims), 0)

  # TODO: auth0_authorize
  # TODO: firebase_authorize



from flask import g, render_template, request, session
from gaelib.auth.models import User
import constants
import random
import string


def generate_user_token(length):
  letters = string.ascii_lowercase
  token = ''.join(random.choice(letters) for i in range(length))
  g.app.logger.info("Token of length " + str(length) + " is: " + token)
  return token


def get_user_token():
  token = request.headers.get('token')
  return token


def verify_request(token):
  g.app.logger.info("Verifying request")
  token_filter = ('token', '=', token)
  try:
    user = User.retrieve(filters=[token_filter])[0]
  except IndexError:
    return None
  return user



from gaelib.tests.auth.base import BaseAuthUnitTestCase
from gaelib.auth.models import User
from gaelib.auth import views
from gaelib.utils import web
from flask import g
import base64
import json

class LoginViewTestCase(BaseAuthUnitTestCase):
  def setUp(self):
    super().setUp()
    self.mock_firebase_authorize()
    self.mock_auth0_authorize()
    self.mock_login_callback()

  def tearDown(self):
    self.firebase_authorize_patch.stop()
    self.auth0_authorize_patch.stop()
    self.login_callback_patch.stop()
    super().tearDown()

  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_not_present_and_user_does_not_exist(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))

  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_not_present_and_user_already_exists(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    user = self.add_user_entity(uid='sub', name='user', email='email@abc')
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user_response = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))
    self.assertEquals(user.key().id, data['id'])


  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_firebase_and_user_does_not_exist(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=firebase', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_firebase_and_user_already_exists(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    user = self.add_user_entity(uid='sub', name='user', email='email@abc')
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=firebase', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user_response = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))
    self.assertEquals(user.key().id, data['id'])


  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_auth_0_and_user_does_not_exist(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.auth0_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=auth0', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_auth_0_and_user_already_exists(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    user = self.add_user_entity(uid='sub', name='user', email='email@abc')
    self.auth0_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=auth0', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user_response = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))
    self.assertEquals(user.key().id, data['id'])


  def test_when_authorization_is_present_with_both_user_id_and_id_token_and_auth_type_is_not_present_and_user_does_not_exist(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_authorization_is_present_with_user_id_and_auth_type_is_not_present(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals("NO_ID_TOKEN", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_when_authorization_is_present_with_id_token_and_auth_type_is_not_present(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b":pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals("NO USER_ID", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))
    self.assertEquals(0, self.get_entity_count('Player'))

  def test_when_authorization_is_not_present_and_auth_type_is_not_present(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals("NO USER_ID", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_not_present_and_user_does_not_exist(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    response = self.client.post('/login/')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_not_present_and_user_already_exists(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    user = self.add_user_entity(uid='sub', name='user', email='email@abc')
    self.firebase_authorize.return_value = claims
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    response = self.client.post('/login/')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user_response = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))
    self.assertEquals(user.key().id, data['id'])


  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_firebase_and_user_does_not_exist(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    self.firebase_authorize.return_value = claims
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    response = self.client.post('/login/?auth_type=firebase')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_firebase_and_user_already_exists(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    user = self.add_user_entity(uid='sub', name='user', email='email@abc')
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=firebase')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user_response = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))
    self.assertEquals(user.key().id, data['id'])


  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_auth_0_and_user_does_not_exist(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    self.auth0_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=auth0')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_auth_0_and_user_already_exists(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    user = self.add_user_entity(uid='sub', name='user', email='email@abc')
    self.auth0_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=auth0')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user_response = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))
    self.assertEquals(user.key().id, data['id'])


  def test_when_session_is_present_with_both_user_id_and_id_token_and_auth_type_is_not_present_and_user_does_not_exist(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
      sess['gae_token'] = 'token'
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    user = User(key_str=data['id'])
    self.assertIsNotNone(user)
    self.assertEquals(data["uId"], user.uid)
    self.assertEquals(data["name"], user.name)
    self.assertEquals(1, self.get_entity_count('User'))


  def test_when_session_is_present_with_user_id_and_auth_type_is_not_present(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    with self.client.session_transaction() as sess:
      sess['gae_uid'] = 'sub'
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals("NO_ID_TOKEN", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_when_session_is_present_with_id_token_and_auth_type_is_not_present(self):
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {}
    }
    with self.client.session_transaction() as sess:
      sess['gae_token'] = 'token'
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/')
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals("NO USER_ID", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_when_claims_is_empty_and_auth_type_is_firebase(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {}
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(401, response.status_code)
    self.assertEquals("EMPTY_CLAIM", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_when_claims_is_empty_and_auth_type_is_auth0(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {}
    self.auth0_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=auth0', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(401, response.status_code)
    self.assertEquals("EMPTY_CLAIM", data["error_message"])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_for_provider_when_auth_type_is_firebase(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"sub:pass").decode("utf8"))
    }
    claims = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {'sign_in_provider': 'google'}
    }
    self.firebase_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=firebase', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    self.assertEquals(data["provider"], 'google')

  def test_for_provider_when_auth_type_is_auth0(self):
    headers = {
        'Authorization': "Basic {}".format(base64.b64encode(b"apple-apple:pass").decode("utf8"))
    }
    claims = {
        'sub': 'apple-apple',
        'name': 'user',
        'email': 'email@abc',
        'firebase': {'sign_in_provider': 'google'}
    }
    self.auth0_authorize.return_value = claims
    response = self.client.post('/login/?auth_type=auth0', headers=headers)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(202, response.status_code)
    self.assertEquals(data["provider"], 'apple')


class VerificationTestCase(BaseAuthUnitTestCase):

  def setUp(self):
    super().setUp()
    self.mock_send_verification_code()
    self.mock_check_verification()
    self.mock_login_callback()
    self.auth_type_json = {'auth_type': 'verify'}

  def tearDown(self):
    self.send_verification_code_patch.stop()
    self.check_verification_patch.stop()
    self.login_callback_patch.stop()
    super().tearDown()

  def test_verification_when_verifying_with_phone_and_verification_code_is_sent(self):
    self.send_verification_code.return_value = "NOT_NONE"
    response = self.client.get("/verification/?phone=1234", json=self.auth_type_json)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(200, response.status_code)
    self.assertEquals('success', data['status'])

  def test_verification_when_verifying_with_phone_and_verification_code_is_not_sent(self):
    self.send_verification_code.return_value = None
    response = self.client.get("/verification/?phone=1234", json=self.auth_type_json)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('Error while sending verification code', data['error_message'])

  def test_verification_when_verifying_with_email_and_verification_code_is_sent(self):
    self.send_verification_code.return_value = "NOT_NONE"
    response = self.client.get("/verification/?email=abcd", json=self.auth_type_json)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(200, response.status_code)
    self.assertEquals('success', data['status'])

  def test_verification_when_verifying_with_email_and_verification_code_is_not_sent(self):
    self.send_verification_code.return_value = None
    response = self.client.get("/verification/?email=abcd", json=self.auth_type_json)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('Error while sending verification code', data['error_message'])

  def test_verification_when_verifying_with_no_method_and_verification_code_is_not_sent(self):
    self.send_verification_code.return_value = None
    response = self.client.get("/verification/", json=self.auth_type_json)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('No phone number or e-mail specified', data['error_message'])

  def test_verification_when_verifying_with_phone_and_correct_code_and_user_does_not_exist(self):
    form_data = {
      'phone': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    self.check_verification.return_value = 'success'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(200, response.status_code)
    self.assertIsNotNone(data['token'])
    self.assertEquals('success', data['status'])
    self.assertEquals(1, self.get_entity_count('User'))
    user = User.retrieve(filters=[('token', '=', data['token'])])[0]
    self.assertEquals('1234', user.phone)


  def test_verification_when_verifying_with_phone_and_correct_code_and_user_exists(self):
    form_data = {
      'phone': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    user = self.add_user_entity(phone='1234')
    self.check_verification.return_value = 'success'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(200, response.status_code)
    self.assertIsNotNone(data['token'])
    self.assertEquals('success', data['status'])
    self.assertEquals(1, self.get_entity_count('User'))
    user_resp = User.retrieve(filters=[('token', '=', data['token'])])[0]
    self.assertEquals('1234', user_resp.phone)
    self.assertEquals(user.key().id, user_resp.key().id)


  def test_verification_when_verifying_with_phone_and_incorrect_code_and_user_does_not_exist(self):
    form_data = {
      'phone': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    self.check_verification.return_value = 'failure'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('Incorrect Code', data['error_message'])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_verification_when_verifying_with_phone_and_incorrect_code_and_user_exists(self):
    form_data = {
      'phone': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    user = self.add_user_entity(phone='1234')
    self.check_verification.return_value = 'failure'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('Incorrect Code', data['error_message'])
    self.assertEquals(1, self.get_entity_count('User'))

  def test_verification_when_verifying_with_email_and_correct_code_and_user_does_not_exist(self):
    form_data = {
      'email': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    self.check_verification.return_value = 'success'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(200, response.status_code)
    self.assertIsNotNone(data['token'])
    self.assertEquals('success', data['status'])
    self.assertEquals(1, self.get_entity_count('User'))
    user = User.retrieve(filters=[('token', '=', data['token'])])[0]
    self.assertEquals('1234', user.email)
    self.assertIsNone(user.phone)


  def test_verification_when_verifying_with_email_and_correct_code_and_user_exists(self):
    form_data = {
      'email': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    user = self.add_user_entity(email='1234', phone=None)
    self.check_verification.return_value = 'success'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(200, response.status_code)
    self.assertIsNotNone(data['token'])
    self.assertEquals('success', data['status'])
    self.assertEquals(1, self.get_entity_count('User'))
    user_resp = User.retrieve(filters=[('token', '=', data['token'])])[0]
    self.assertEquals('1234', user_resp.email)
    self.assertIsNone(user_resp.phone)
    self.assertEquals(user.key().id, user_resp.key().id)


  def test_verification_when_verifying_with_email_and_incorrect_code_and_user_does_not_exist(self):
    form_data = {
      'email': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    self.check_verification.return_value = 'failure'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('Incorrect Code', data['error_message'])
    self.assertEquals(0, self.get_entity_count('User'))

  def test_verification_when_verifying_with_email_and_incorrect_code_and_user_exists(self):
    form_data = {
      'email': '1234',
      'code': '1234',
      'auth_type': 'verify'
    }
    user = self.add_user_entity(phone='1234')
    self.check_verification.return_value = 'failure'
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('Incorrect Code', data['error_message'])
    self.assertEquals(1, self.get_entity_count('User'))

  def test_verification_when_verifying_with_no_method(self):
    form_data = {
      'code': '1234',
      'auth_type': 'verify'
    }
    response = self.client.post('/verification/', data=form_data)
    data = json.loads(response.get_data(as_text=True))
    self.assertEquals(400, response.status_code)
    self.assertEquals('No Phone Number', data['error_message'])
    self.assertEquals(0, self.get_entity_count('User'))

class ViewsTestCase(BaseAuthUnitTestCase):

  def test_check_for_new_user_with_uid_when_new_user(self):
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = views.check_for_new_user_with_uid(g.app.logger, 'user_id', {'name': 'user', 'email': 'user@cc'}, None)
      self.assertIsNotNone(user)
      self.assertEquals(1, self.get_entity_count('User'))
      self.assertEquals('user_id', user.uid)
      self.assertEquals('user', user.name)

  def test_check_for_new_user_if_device_token_updated_with_uid_when_new_user(self):
    device_token_data = {
      'token': 'test_token',
      'os': 'test_os'}
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = views.check_for_new_user_with_uid(g.app.logger, 'user_id', {'name': 'user', 'email': 'user@cc'}, device_token_data)
      self.assertIsNotNone(user)
      self.assertEquals(1, self.get_entity_count('User'))
      self.assertEquals('user_id', user.uid)
      self.assertEquals('user', user.name)
      self.assertEquals('test_token', user.device_notification_token)
      self.assertEquals('test_os', user.os)

  def test_check_for_new_user_when_existing_user(self):
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(uid='user_id')
      self.assertEquals(1, self.get_entity_count('User'))
      user_resp = views.check_for_new_user_with_uid(g.app.logger, 'user_id', {'name': 'user', 'email': 'user@cc'}, None)
      self.assertIsNotNone(user)
      self.assertEquals(1, self.get_entity_count('User'))
      self.assertEquals(user.key().id, user_resp.key().id)
      self.assertEquals('user_id', user_resp.uid)
      self.assertEquals('user', user_resp.name)

  def test_check_for_new_user_if_device_token_updated_when_existing_user(self):
    device_token_data = {
      'token': 'test_token',
      'os': 'test_os'}
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(uid='user_id')
      self.assertEquals(1, self.get_entity_count('User'))
      user_resp = views.check_for_new_user_with_uid(g.app.logger, 'user_id', {'name': 'user', 'email': 'user@cc'}, device_token_data)
      self.assertIsNotNone(user)
      self.assertEquals(1, self.get_entity_count('User'))
      self.assertEquals(user.key().id, user_resp.key().id)
      self.assertEquals('user_id', user_resp.uid)
      self.assertEquals('user', user_resp.name)
      self.assertEquals('test_token', user_resp.device_notification_token)
      self.assertEquals('test_os', user_resp.os)

  def test_check_for_new_user_with_phone_or_email_with_phone_or_email_when_new_user_and_method_is_phone(self):
    user = self.add_user_entity(phone='0183409112', token='abc')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(
          g.app.logger, '0183409', None, None)
      self.assertIsNotNone(result)
      self.assertEquals('0183409', result.phone)
      self.assertEquals(0, result.role)

  def test_check_for_new_user_with_phone_or_email_with_phone_or_email_when_new_user_and_method_is_email(self):
    user = self.add_user_entity(email='email123@abc.com', token='abc')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, None, 'email@abc.com', None)
      self.assertIsNotNone(result)
      self.assertEquals('email@abc.com', result.email)
      self.assertEquals(0, result.role)

  def test_check_for_new_user_when_new_user_and_method_is_None(self):
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, None, None, None)
      self.assertIsNone(result)

  def test_check_for_new_user_with_phone_or_email_when_existing_user_and_method_is_phone(self):
    user = self.add_user_entity(phone='0183409', token='abc')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, '0183409', None, None)
      self.assertIsNotNone(result)
      self.assertEquals('0183409', result.phone)
      self.assertEquals(user.key().id, result.key().id)
      self.assertNotEquals(user.token, result.token)

  def test_check_for_new_user_with_phone_or_email_when_existing_user_and_method_is_email(self):
    user = self.add_user_entity(email='email@abc.com', token='abc')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, None, 'email@abc.com', None)
      self.assertIsNotNone(result)
      self.assertEquals('email@abc.com', result.email)
      self.assertEquals(user.key().id, result.key().id)
      self.assertNotEquals(user.token, result.token)

  def test_check_for_new_user_with_phone_or_email_if_device_token_updated_when_existing_user_and_method_is_email(self):
    user = self.add_user_entity(email='email@abc.com', token='abc')
    device_token_data = {
      'token': 'test_token',
      'os': 'test_os'}
    with self.tests_app.test_request_context(json={'device_token_data': device_token_data}):
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, None, 'email@abc.com', device_token_data)
      self.assertIsNotNone(result)
      self.assertEquals('email@abc.com', result.email)
      self.assertEquals(user.key().id, result.key().id)
      self.assertEquals(result.os, 'test_os')
      self.assertEquals(result.device_notification_token, 'test_token')
      self.assertNotEquals(user.token, result.token)
  
  def test_check_for_new_user_with_phone_or_email_if_device_token_updated_when_existing_user_and_method_is_phone(self):
    user = self.add_user_entity(phone='12345', token='abc')
    device_token_data = {
      'token': 'test_token',
      'os': 'test_os'}
    with self.tests_app.test_request_context(json={'device_token_data': device_token_data}):
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, '12345', None, device_token_data)
      self.assertIsNotNone(result)
      self.assertEquals('12345', result.phone)
      self.assertEquals(user.key().id, result.key().id)
      self.assertEquals(result.os, 'test_os')
      self.assertEquals(result.device_notification_token, 'test_token')
      self.assertNotEquals(user.token, result.token)

  def test_check_for_new_user_with_phone_or_email_if_device_token_updated_when_not_existing_user_and_method_is_email(self):
    device_token_data = {
      'token': 'test_token',
      'os': 'test_os'}
    with self.tests_app.test_request_context(json={'device_token_data': device_token_data}):
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, None, 'email@abc.com', device_token_data)
      self.assertIsNotNone(result)
      self.assertEquals('email@abc.com', result.email)
      self.assertEquals(result.os, 'test_os')
      self.assertEquals(result.device_notification_token, 'test_token')
  
  def test_check_for_new_user_with_phone_or_email_if_device_token_updated_when_not_existing_user_and_method_is_phone(self):
    device_token_data = {
      'token': 'test_token',
      'os': 'test_os'}
    with self.tests_app.test_request_context(json={'device_token_data': device_token_data}):
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, '12345', None, device_token_data)
      self.assertIsNotNone(result)
      self.assertEquals('12345', result.phone)
      self.assertEquals(result.os, 'test_os')
      self.assertEquals(result.device_notification_token, 'test_token')

  def test_check_for_new_user_with_phone_or_email_for_profile_picture_when_not_existing_user(self):
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, '12345', None, None)
      self.assertIsNotNone(result)
      self.assertEquals('12345', result.phone)
      self.assertEquals('https://storage.googleapis.com/crypticcup-images/default_profile.jpg', result.picture)

  def test_check_for_new_user_with_phone_or_email_for_profile_picture_when_existing_user(self):
    user = self.add_user_entity(phone='12345', token='abc', picture='picture@')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      result = views.check_for_new_user_with_phone_or_email(g.app.logger, '12345', None, None)
      self.assertIsNotNone(result)
      self.assertEquals('12345', result.phone)
      self.assertEquals('picture@', result.picture)

  def test_check_for_new_user_when_auth_type_is_verify(self):
    self.mock_check_for_new_user_with_phone_or_email()
    self.mock_check_for_new_user_with_uid()
    with self.tests_app.test_request_context('/url/?auth_type=verify'):
      self.tests_app.preprocess_request()
      views.check_for_new_user(g.app.logger)
      self.assertTrue(self.check_for_new_user_with_phone_or_email.called)
      self.assertFalse(self.check_for_new_user_with_uid.called)
    self.check_for_new_user_with_phone_or_email_patch.stop()
    self.check_for_new_user_with_uid_patch.stop()

  def test_check_for_new_user_when_auth_type_is_not_verify(self):
    self.mock_check_for_new_user_with_phone_or_email()
    self.mock_check_for_new_user_with_uid()
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      views.check_for_new_user(g.app.logger)
      self.assertFalse(self.check_for_new_user_with_phone_or_email.called)
      self.assertTrue(self.check_for_new_user_with_uid.called)
    self.check_for_new_user_with_phone_or_email_patch.stop()
    self.check_for_new_user_with_uid_patch.stop()





from twilio.rest import Client
from gaelib.env import get_twilio_account_sid, get_twilio_auth_token, get_twilio_verification_sid
from gaelib.utils.web import app


class TwilioClient():
  __client__ = None

  @classmethod
  def get_client(self):
    # To maintain Singleton State of the Client
    if self.__client__ == None:
      self.__client__ = Client(app.config.get(
          'ACCOUNT_SID'), app.config.get('AUTH_TOKEN'))
    return self.__client__

  @classmethod
  def send_verification_code(self, to, channel):
    service = app.config.get('VERIFICATION_SID')
    client = self.get_client()
    verification = client.verify \
        .services(service) \
        .verifications \
        .create(to=to, channel=channel)
    return verification.sid

  @classmethod
  def check_verification(self, phone, code):
    service = app.config.get('VERIFICATION_SID')
    client = self.get_client()
    try:
      verification_check = client.verify \
          .services(service) \
          .verification_checks \
          .create(to=phone, code=code)
    except:
      return 'failure'
    return 'success' if verification_check.status == 'approved' else 'failure'



"""
    This module defines the testcases for the Model class.
"""
from gaelib.tests.base import BaseUnitTestCase

from .model import Test


class ModelUnitTestCase(BaseUnitTestCase):
    
    longMessage = False

    def create_object(self,**kwargs):
        obj = Test(**kwargs)
        return obj

    def test_passing_an_already_set_key_behaviour(self):
        #adding a key in the db to check with
        key = Test.generate_key('set_key_behaviour')
        model = Test(key = key, string1 = 'value1')
        model.put()      

        #passing the same key again to see the behaviour
        model = Test(key = key)
        self.assertTrue(model.retrieved(),"Key not found") 
    
    def test_key_str_behaviour(self):
        model = Test(key_str = 'key_str_behaviour')
        self.assertIsNotNone(model.key(), "Failed to generate key from key_str")
    
    def test_entity_as_keyword_behaviour(self): 
        model = Test(entity = self.create_object(string1 = 'value1', string2 = 'value2'))
        self.assertIsNotNone(model.__entity__, "Failed to create entity")

    def test_no_id_behaviour(self):
        model = Test()
        self.assertIsNotNone(model.key(), "Failed to generate key")
    
    def test_check_default_properties(self):
        model = self.create_object()
        attributes = ['string1','string2', 'int1', 'float1', 'bool1']
        for attribute in attributes:
            self.assertIsNone(model.__getattribute__(attribute), "Default value not none for attribute: {}".format(attribute))

    def check_keywords_passed(self, **kwargs):
        model = Test(**kwargs)
        model.put()
        for key, val in kwargs.items():
            self.assertEqual(model.__getattribute__(key),val, "Failed for key-value pair {} : {}".format(key,val))
      
    def test_check_keywords_passed(self):
        self.check_keywords_passed(string1 = 'one', string2 = 'two')

    def test_check_validation_of_props(self):
        self.assertRaises(ValueError, self.create_object, string1 = '123', string2 = 123, int1 = 123.40, float1 = True, bool1 = 123)

    def test_check_update_and_put(self):
        #checking put
        key = Test.generate_key('put and update')
        self.clear_database()

        model = Test(key = key, string1 = 'test_set_1')
        model.put()
        #model.retrieved() won't work here because the above instance of model is for creating the entity
        #and in that case self.__retrieved__ is set to false
        #We need to create another instance of the model with the same key and
        #then check for model.retrieved()
        model = Test(key = key)
        self.assertTrue(model.retrieved(),"Put failed")

        #checking update
        model.update(string1 = 'test_set_2')
        model.put()

        self.assertEqual(model.__getattribute__('string1'), 'test_set_2', "Update Failed")
        
    def test_check_delete(self):
        key = Test.generate_key('delete')
        #creatng a record to be deleted
        model = Test(key = key, string1 = "This record will be deleted")
        model.put()
        model = Test(key = key)
        self.assertTrue(model.retrieved(), "Failed to create entity")
        model.delete()
        model = Test(key = key)
        self.assertFalse(model.retrieved(), "Failed to delete key")
        
    def test_query(self):
        #create records to be fetched
        model = Test(key_str = 'record1', string1 = '123', string2 = '456', int1 = 111)
        model.put()
        model = Test(key_str = 'record2', string1 = '123', string2 = '456', int1 = 113)
        model.put()
        model = Test(key_str = 'record3', string1 = '123', string2 = '456', int1 = 112)
        model.put()

        #fetch records
        model = Test()
        q = model.query()
        q.add_filter('string1', '=', '123')
        q.add_filter('int1', '>', 111)
        q.assign_order('-int1') #descending
        last_value = 200
        for obj in q.fetch():
            self.assertGreaterEqual(last_value,obj.int1, "Failed to order items in result")
            last_value = obj.int1
        self.assertEquals(len(q.fetch()), 2, "Failed to fetch. Fetched {} instead of 2 items".format(len(q.fetch())))

    def test_retrieve(self):
        model = Test(key_str = 'record1', string1 = '123', string2 = '456', int1 = 111)
        model.put()
        model = Test(key_str = 'record2', string1 = '123', string2 = '456', int1 = 112)
        model.put()
        model = Test(key_str = 'record3', string1 = '123', string2 = '456', int1 = 113)
        model.put()
        model = Test(key_str = 'record3', string1 = '123', string2 = '456', int1 = 114)
        model.put()

        model = Test()

        #testing for key_str        
        items = model.retrieve(key_strs = ['record1'])
        self.assertEquals(items[0].int1, 111, "Failed to retrieve using key_str")

        #testing for filters, order and limit
        limit = 2
        items = model.retrieve(filters=[('string1', '=', '123'), ('int1', '>', 111)], order = '-int1', limit = limit)
        last_value = 200
        for obj in items:
            self.assertGreater(obj.int1, 111, "Failed to apply filter")
            self.assertEquals(obj.string1,'123', "Failed to apply filter")
            self.assertGreaterEqual(last_value,obj.int1, "Failed to order items")
            last_value = obj.int1
        self.assertEquals(len(items),limit, "Failed to apply limit. Fetched {} instead of {} items".format(len(items),limit))



import os

from flask import g, jsonify, redirect, request, session
import constants
import gaelib.defaults as defaults


def is_dev():
  if os.getenv('GAE_DEPLOYMENT_ID', ''):
    return False
  else:
    return True


def is_prod():
  return not is_dev() and not is_staging()


def is_staging():
  return not is_dev() and 'staging' in os.getenv('GOOGLE_CLOUD_PROJECT')


def get_app_or_default_prop(prop_name):
  try:
    prop_value = getattr(constants, prop_name)
  except (AttributeError, ModuleNotFoundError):
    prop_value = ''
  if not prop_value:
    prop_value = getattr(defaults, prop_name)

  return prop_value


def get_dashboard_url_prefix():
  return get_app_or_default_prop('DASHBOARD_URL_PREFIX')


def get_dashboard_assets_prefix():
  return get_app_or_default_prop('DASHBOARD_ASSETS_PREFIX')


def is_dashboard_url():
  url = request.path
  return url.strip('/').startswith(get_dashboard_url_prefix())


def get_post_login_page():
  return get_app_or_default_prop('POST_LOGIN_PAGE')


def get_sidebar_template():
  return get_app_or_default_prop('SIDEBAR_TEMPLATE')


def get_staff_phones():
  return get_app_or_default_prop('STAFF_PHONES')


def get_profile_picture():
  return get_app_or_default_prop('DEFAULT_PROFILE_IMAGE')


def get_token_length():
  return get_app_or_default_prop('TOKEN_LENGTH')


def get_twilio_verification_sid():
  return get_app_or_default_prop('VERIFICATION_SID')


def get_twilio_account_sid():
  return get_app_or_default_prop('ACCOUNT_SID')


def get_twilio_auth_token():
  return get_app_or_default_prop('AUTH_TOKEN')


def dashboard_login_page():
  return redirect("/" + get_dashboard_url_prefix() + '/login', code=302)


def get_env():
  env = 'staging' if not is_prod() else 'prod'
  return env



from gaelib.tests.auth.base import BaseAuthUnitTestCase
from gaelib.auth.decorators import auth_required, verification_required, get_login_user, access_control
from gaelib.auth.models import User, UserRole
from gaelib.utils import web
from mock import Mock
from flask import request
import json


class DecoratorsTestCase(BaseAuthUnitTestCase):

  def setUp(self):
    super().setUp()
    self.mock_get_user_id_and_token()
    self.mock_firebase_authorize()
    self.mock_get_user_token()
    self.mock_get_auth_type()

  def tearDown(self):
    self.get_user_id_and_token_patch.stop()
    self.firebase_authorize_patch.stop()
    self.get_user_token_patch.stop()
    self.get_auth_type_patch.stop()
    super().tearDown()

  def test_auth_required_when_user_id_is_not_present_and_id_token_is_not_present_and_is_dashboard_url(self):
    self.get_user_id_and_token.return_value = (
        None, None)
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      self.assertEquals(302, response.status_code)
      self.assertEquals('/admindashboard/login', response.location)

  def test_auth_required_when_user_id_is_not_present_and_id_token_is_present_and_is_dashboard_url(self):
    self.get_user_id_and_token.return_value = (
        None, 'test_token')
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      self.assertEquals(302, response.status_code)
      self.assertEquals('/admindashboard/login', response.location)

  def test_auth_required_when_user_id_is_present_and_id_token_is_not_present_and_is_dashboard_url(self):
    self.get_user_id_and_token.return_value = (
        'test_user', None)
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      self.assertEquals(302, response.status_code)
      self.assertEquals('/admindashboard/login', response.location)

  def test_auth_required_when_user_id_is_present_and_id_token_is_present_and_is_dashboard_url(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    self.firebase_authorize.return_value = return_value
    self.get_user_id_and_token.return_value = (
        'sub', 'test_token')
    self.get_auth_type.return_value = 'firebase'
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      user = self.add_user_entity(uid='sub', name='user')
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      self.assertEquals("OK", response)

  def test_auth_required_when_user_id_is_not_present_and_id_token_is_not_present_and_not_is_dashboard_url(self):
    self.get_user_id_and_token.return_value = (
        None, None)
    with self.tests_app.test_request_context():
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED USER', data['error_message'])

  def test_auth_required_when_user_id_is_not_present_and_id_token_is_present_and_is_not_dashboard_url(self):
    self.get_user_id_and_token.return_value = (
        None, 'test_token')
    with self.tests_app.test_request_context():
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED USER', data['error_message'])

  def test_auth_required_when_user_id_is_present_and_id_token_is_not_present_and_is_not_dashboard_url(self):
    self.get_user_id_and_token.return_value = (
        'test_user', None)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEqual('UNAUTHORIZED TOKEN', data['error_message'])

  def test_auth_required_when_user_id_is_present_and_id_token_is_present_and_is_not_dashboard_url(self):
    return_value = {
        'sub': 'sub',
        'name': 'user',
        'email': 'email@abc'
    }
    self.firebase_authorize.return_value = return_value
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      self.get_user_id_and_token.return_value = (
          'test_user', 'test_token')
      decorated_func = auth_required(Mock(return_value="OK"))
      response = decorated_func()

  def test_verification_required_when_token_is_absent_and_is_dashboard_url(self):
    self.get_user_token.return_value = (None)
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      decorated_func = verification_required(Mock(return_value="OK"))
      response = decorated_func()
      self.assertEquals(302, response.status_code)
      self.assertEquals('/admindashboard/login', response.location)

  def test_verification_required_when_token_is_present_and_is_not_dashboard_url(self):
    self.get_user_token.return_value = ('test_token')
    decorated_func = verification_required(Mock(return_value="OK"))
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      response = decorated_func()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEqual('Unauthorised login for web request found',
                       data['error_message'])

  def test_verification_required_when_token_is_absent_and_is_not_dashboard_url(self):
    self.get_user_token.return_value = (None)
    decorated_func = verification_required(Mock(return_value="OK"))
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      response = decorated_func()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEqual('USER TOKEN NOT FOUND IN HEADERS',
                       data['error_message'])

  def test_verification_required_when_token_is_present_and_is_dashboard_url_and_user_with_token_is_present(self):
    self.get_user_token.return_value = ('test_token')
    decorated_func = verification_required(Mock(return_value="OK"))
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      user = self.add_user_entity(token='test_token')
      response = decorated_func()
      self.assertEquals("OK", response)

  def test_verification_required_when_token_is_present_and_is_not_dashboard_url_and_user_with_token_is_present(self):
    self.get_user_token.return_value = ('test_token')
    decorated_func = verification_required(Mock(return_value="OK"))
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      user = self.add_user_entity(token='test_token')
      response = decorated_func()
      self.assertEquals("OK", response)

  def test_verification_required_when_token_is_present_and_is_dashboard_url_and_user_with_token_is_absent(self):
    self.get_user_token.return_value = ('test_token')
    decorated_func = verification_required(Mock(return_value="OK"))
    with self.tests_app.test_request_context('/admindashboard/login'):
      self.tests_app.preprocess_request()
      response = decorated_func()
      self.assertEquals(302, response.status_code)
    self.assertEquals('/admindashboard/login', response.location)

  def test_login_user_when_auth_type_is_firebase(self):
    self.get_auth_type.return_value = 'firebase'
    self.get_user_token.return_value = ('token_1')
    self.get_user_id_and_token.return_value = ('id_2', None)
    with self.tests_app.test_request_context():
      user1 = User(uid='id_1', token='token_1')
      user2 = User(uid='id_2', token='token_2')
      user1.put()
      user2.put()
      self.tests_app.preprocess_request()
      response = get_login_user()
      self.assertEquals(user2.key().id, response.key().id)

  def test_login_user_when_auth_type_is_auth0(self):
    self.get_auth_type.return_value = 'auth0'
    self.get_user_token.return_value = ('token_1')
    self.get_user_id_and_token.return_value = ('id_2', None)
    with self.tests_app.test_request_context():
      user1 = User(uid='id_1', token='token_1')
      user2 = User(uid='id_2', token='token_2')
      user1.put()
      user2.put()
      self.tests_app.preprocess_request()
      response = get_login_user()
      self.assertEquals(user2.key().id, response.key().id)

  def test_login_user_when_auth_type_is_verify(self):
    self.get_auth_type.return_value = 'verify'
    self.get_user_token.return_value = ('token_1')
    self.get_user_id_and_token.return_value = ('id_2', None)
    with self.tests_app.test_request_context():
      user1 = User(uid='id_1', token='token_1')
      user2 = User(uid='id_2', token='token_2')
      user1.put()
      user2.put()
      self.tests_app.preprocess_request()
      response = get_login_user()
      self.assertEquals(user1.key().id, response.key().id)

  def test_access_control_when_user_is_staff_member_and_auth_type_is_firebase(self):
    self.get_auth_type.return_value = 'firebase'
    self.get_user_id_and_token.return_value = ('id_1', None)
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(role=1, uid='id_1')
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      self.assertEquals("OK", response)

  def test_access_control_when_user_is_staff_member_and_auth_type_is_auth0(self):
    self.get_auth_type.return_value = 'auth0'
    self.get_user_id_and_token.return_value = ('id_1', None)
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(role=1, uid='id_1')
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      self.assertEquals("OK", response)

  def test_access_control_when_user_is_staff_member_and_auth_type_is_verify(self):
    self.get_auth_type.return_value = 'verify'
    self.get_user_token.return_value = ('token_1')
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(role=1, token='token_1')
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      self.assertEquals("OK", response)

  def test_access_control_when_user_is_non_staff_member_and_auth_type_is_firebase(self):
    self.get_auth_type.return_value = 'firebase'
    self.get_user_id_and_token.return_value = ('id_1', None)
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(role=0, uid='id_1')
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED ACCESS',
                        data['error_message'])

  def test_access_control_when_user_is_non_staff_member_and_auth_type_is_auth0(self):
    self.get_auth_type.return_value = 'auth0'
    self.get_user_id_and_token.return_value = ('id_1', None)
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(role=0, uid='id_1')
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED ACCESS',
                        data['error_message'])

  def test_access_control_when_user_is_non_staff_member_and_auth_type_is_verify(self):
    self.get_auth_type.return_value = 'verify'
    self.get_user_token.return_value = ('token_1')
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = self.add_user_entity(role=0, token='token_1')
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED ACCESS',
                        data['error_message'])

  def test_access_control_when_user_does_not_exist_and_auth_type_is_firebase(self):
    self.get_auth_type.return_value = 'firebase'
    self.get_user_id_and_token.return_value = ('id_1', None)
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED ACCESS',
                        data['error_message'])

  def test_access_control_when_user_does_not_exist_and_auth_type_is_auth0(self):
    self.get_auth_type.return_value = 'auth0'
    self.get_user_id_and_token.return_value = ('id_1', None)
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED ACCESS',
                        data['error_message'])

  def test_access_control_when_user_does_not_exist_and_auth_type_is_verify(self):
    self.get_auth_type.return_value = 'verify'
    self.get_user_token.return_value = ('token_1')
    decorated_func = access_control(role=UserRole.STAFF)
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      decorated_func_1 = decorated_func(Mock(return_value="OK"))
      response = decorated_func_1()
      data = json.loads(response[0].get_data(as_text=True))
      self.assertEquals('UNAUTHORIZED ACCESS',
                        data['error_message'])



from gaelib.tests.auth.base import BaseAuthUnitTestCase
from gaelib.storage import helpers
from gaelib.auth.twilio_client import TwilioClient
from gaelib.auth import verify
from gaelib.auth.models import User
from mock import patch
from gaelib.utils import web
from flask import g


class VerifyTestCase(BaseAuthUnitTestCase):

  def setUp(self):
    super().setUp()
    self.mock_twilio_client()

  def test_generate_user_token(self):
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      token = verify.generate_user_token(10)
      self.assertEquals(len(token), 10)
      self.assertTrue(token.islower())

  def test_get_user_token(self):
    url = 'admindashboard/login'
    headers = {
        'token': 'dummy_token'
    }
    with self.tests_app.test_request_context(url, headers=headers):
      self.tests_app.preprocess_request()
      token = verify.get_user_token()
      self.assertEquals(token, 'dummy_token')

  def test_get_user_token_when_no_token_is_sent(self):
    url = 'admindashboard/login'
    with self.tests_app.test_request_context(url):
      self.tests_app.preprocess_request()
      token = verify.get_user_token()
      self.assertIsNone(token)

  def test_verify_request_when_token_matches(self):
    user = self.add_user_entity(name='User', token='Token')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = verify.verify_request('Token')
      self.assertIsNotNone(user)
      self.assertEquals(user.name, 'User')

  def test_verify_request_when_token_does_not_match(self):
    user = self.add_user_entity(name='User', token='Token')
    with self.tests_app.test_request_context():
      self.tests_app.preprocess_request()
      user = verify.verify_request('Token_1')
      self.assertIsNone(user)



"""
    This module contains all defaults available for when
    the application importing the lib doesnt set values
"""
DASHBOARD_URL_PREFIX = 'admindashboard'
DASHBOARD_ASSETS_PREFIX = 'https://storage.googleapis.com/gaelib-assets/assets'
POST_LOGIN_PAGE = 'users'
SIDEBAR_TEMPLATE = 'dashboard/operations_sidebar'
PARAMETER_LOGGING = 'true'
SESSION_SECRET = 'lib_key'
# TODO: Move this to a more generic bucket name
DEFAULT_PROFILE_IMAGE = 'https://storage.googleapis.com/crypticcup-images/default_profile.jpg'

# Twilio Auth Settings
VERIFICATION_SID = ''
ACCOUNT_SID = ''
AUTH_TOKEN = ''
TOKEN_LENGTH = 12



from mock import patch
from gaelib.auth.models import User
from gaelib.tests.base import BaseUnitTestCase
from gaelib.utils import web


class BaseAuthUnitTestCase(BaseUnitTestCase):

  def setUp(self):
    super().setUp()

  def tearDown(self):
    super().tearDown()

  def add_user_entity(self, name='', email='', phone='', picture='http://dp.com', uid='', token='', role=0):
    user = User()
    user.update(
        name=name,
        email=email,
        picture=picture,
        phone=phone,
        uid=uid,
        token=token,
        role=role
    )
    user.put()
    return user

  def mock_twilio_client(self):
    self.twilio_client_patch = patch(
        'gaelib.auth.twilio_client.TwilioClient.get_client')
    self.twilio_client = self.twilio_client_patch.start()

  def mock_firebase_authorize(self):
    self.firebase_authorize_patch = patch(
        'gaelib.auth.auth.Auth.firebase_authorize')
    self.firebase_authorize = self.firebase_authorize_patch.start()

  def mock_auth0_authorize(self):
    self.auth0_authorize_patch = patch(
        'gaelib.auth.auth.Auth.auth0_authorize')
    self.auth0_authorize = self.auth0_authorize_patch.start()

  def mock_auth_obj(self):
    self.auth_obj_patch = patch(
        'gaelib.auth.auth.Auth')
    self.auth_obj = self.auth_obj_patch.start()

  def mock_get_user_id_and_token(self):
    self.get_user_id_and_token_patch = patch(
        'gaelib.auth.auth.get_user_id_and_token')
    self.get_user_id_and_token = self.get_user_id_and_token_patch.start()

  def mock_get_user_token(self):
    self.get_user_token_patch = patch(
        'gaelib.auth.verify.get_user_token')
    self.get_user_token = self.get_user_token_patch.start()

  def mock_get_auth_type(self):
    self.get_auth_type_patch = patch('gaelib.auth.auth.get_auth_type')
    self.get_auth_type = self.get_auth_type_patch.start()

  def mock_send_verification_code(self):
    self.send_verification_code_patch = patch(
        'gaelib.auth.twilio_client.TwilioClient.send_verification_code')
    self.send_verification_code = self.send_verification_code_patch.start()

  def mock_check_verification(self):
    self.check_verification_patch = patch(
        'gaelib.auth.twilio_client.TwilioClient.check_verification')
    self.check_verification = self.check_verification_patch.start()

  def mock_check_for_new_user_with_uid(self):
    self.check_for_new_user_with_uid_patch = patch(
        'gaelib.auth.views.check_for_new_user_with_uid')
    self.check_for_new_user_with_uid = self.check_for_new_user_with_uid_patch.start()

  def mock_check_for_new_user_with_phone_or_email(self):
    self.check_for_new_user_with_phone_or_email_patch = patch(
        'gaelib.auth.views.check_for_new_user_with_phone_or_email')
    self.check_for_new_user_with_phone_or_email = self.check_for_new_user_with_phone_or_email_patch.start()

  def mock_login_callback(self):
    self.login_callback_patch = patch(
        'gaelib.auth.auth.login_callback', create=True)
    self.login_callback = self.login_callback_patch.start()
